(function(window) {
	var Piece = function(canvas, config)
	{
		this.initialize(canvas, config);
	}
	var p = Piece.prototype = new BasePiece();
	//
	p.initialize = function(canvas, config)
	{
		BasePiece.prototype.initialize.call(this, canvas, config);
		this.container = this.stage.addChild(new Container());
		this.shape = this.container.addChild(new Shape());
		this.shapeDebug = this.container.addChild(new Shape());
	}


	p.onKeyUp = function(e)
	{
		BasePiece.prototype.onKeyUp.call(this, e);
		if (!this.config.debug) return;
		var c = String.fromCharCode(e.which);
		if (c=="R") this.reset();
		if (c=="D") this.debugLines = !this.debugLines;
	}
	
	p.setSize = function(w,h,dpr)
	{
		this.dpr = dpr;
		w = this.width = Math.floor(w*dpr);
		h = this.height = Math.floor(h*dpr);
		log("setSize",w,h);
		this.stage.x = w/2;
		this.stage.y = h/2;
		var cfg = this.config;
		//
		var ratio = Math.max(1/cfg.ratioMax, Math.min(cfg.ratioMax, w/h));
		var r = R, target = this.container;
		if (w>h)
		{
			target.scaleY = h/r;
			target.scaleX = ratio * target.scaleY;
		}
		else
		{
			target.scaleX = w/r;
			target.scaleY = target.scaleX/ratio;
		}
		//
		this.scale = Math.min(target.scaleX,target.scaleY);
	}

	p.start = function()
	{
		BasePiece.prototype.start.apply(this);
		log("start",this.width);		
		this.xys = [];
		this.reset();
	}

	p.reset = function()
	{
		//read angles from points in cfg
		var cfg = this.config;
		var xys = cfg.shapeXYs, n = xys.length;
		var verts = [];
		var d = RandomUtil.rnd(cfg.duration);
		for (var i=0;i<n;i+=2)
		{
			var x = xys[i], y = xys[i+1], outer = i%4==0;
			var angle = Math.atan2(y,x), len = Math.sqrt(x*x+y*y);
			var rMin = RandomUtil.rnd(cfg.radiusMin);
			var rMax = RandomUtil.rnd(cfg.radiusMax);
			//
			var t = i%4==0? .5 : 0;
			//
			if (outer) rMax = len;
			else rMin = len;
			//
			var v = {a:angle, d:d, t:t, rMin:rMin, rMax:rMax};
			verts.push(v);
		}
		this.vertices = verts;
	}
	
	
	p.draw = function()
	{
		var cfg = this.config
		var g = this.shape.graphics.c().f(cfg.color);	
		var xys = this.xys, n = xys.length;
		g.mt(xys[n-2],xys[n-1]);
		for (var i=0;i<n;i+=2)
		{
			g.lt(xys[i],xys[i+1]);
		}
		//
		g = this.shapeDebug.graphics.c();
		if (this.debugLines)
		{
			var r = R;
			g.s("#f00").ss(1/this.scale);
			for (var i=0;i<this.vertices.length;i++)
			{
				var v = this.vertices[i];
				var x1 = r*Math.cos(v.a), y1 = r*Math.sin(v.a);
				g.mt(v.rMax*x1,v.rMax*y1).lt(v.rMin*x1,v.rMin*y1);
			}
		}
	}

	
	p.update = function()
	{
		var dt = this.tickDuration;
		//update vertices + calc current points
		var xys = this.xys, s = R;
		xys.length = 0;
		for (var i=0;i<this.vertices.length;i++)
		{
			var v = this.vertices[i];
			v.t += dt/v.d;
			var t = v.t * PI2;
			var t2 = .5*Math.cos(t)+.5;
			var r = s * MathUtil.lerp(v.rMin,v.rMax,t2);
			var x = Math.cos(v.a) * r;
			var y = Math.sin(v.a) * r;
			xys.push(x,y);
		}
		//
		this.draw();
		return true;
	}

	var R = 1;
	var PI2 = Math.PI*2;
	
	window.Piece = Piece;
}(window));

